@@ -8,7 +8,8 @@
 from xpra.client.keyboard_helper import KeyboardHelper, log
 from xpra.gtk_common.gobject_compat import import_gdk, import_glib
 from xpra.gtk_common.keymap import get_gtk_keymap
-from xpra.gtk_common.gtk_util import display_get_default
+from xpra.gtk_common.gtk_util import display_get_default, keymap_get_for_display
+from xpra.os_util import is_X11
 gdk = import_gdk()
 glib = import_glib()
 
@@ -21,10 +22,8 @@ class GTKKeyboardHelper(KeyboardHelper):
         #(as we may be getting dozens of such events at a time)
         self._keymap_changing = False
         self._keymap_change_handler_id = None
-        try:
-            self._keymap = gdk.keymap_get_default()
-        except:
-            self._keymap = None
+        display = display_get_default()
+        self._keymap = keymap_get_for_display(display)
         self.update()
         if self._keymap:
             self._keymap_change_handler_id = self._keymap.connect("keys-changed", self.keymap_changed)
@@ -34,7 +33,8 @@ class GTKKeyboardHelper(KeyboardHelper):
         if self._keymap_change_handler_id:
             self._keymap.disconnect(self._keymap_change_handler_id)
             self._keymap_change_handler_id = None
-        self._keymap = gdk.keymap_get_default()
+        display = display_get_default()
+        self._keymap = keymap_get_for_display(display)
         if self._keymap_changing:
             #timer due already
             return
@@ -57,11 +57,12 @@ class GTKKeyboardHelper(KeyboardHelper):
     def update(self):
         old_hash = self.hash
         self.query_xkbmap()
-        try:
-            self.keyboard.update_modifier_map(display_get_default(), self.xkbmap_mod_meanings)
-        except:
-            log.error("error querying modifier map", exc_info=True)
-        log("do_keys_changed() modifier_map=%s, old hash=%s, new hash=%s", self.keyboard.modifier_map, old_hash, self.hash)
+        if is_X11():
+            try:
+                self.keyboard.update_modifier_map(display_get_default(), self.xkbmap_mod_meanings)
+            except:
+                log.error("error querying modifier map", exc_info=True)
+        log("update() modifier_map=%s, old hash=%s, new hash=%s", self.keyboard.modifier_map, old_hash, self.hash)
         return old_hash!=self.hash
 
     def get_full_keymap(self):
@@ -79,8 +80,10 @@ class GTKKeyboardHelper(KeyboardHelper):
 
 def main():
     #use gtk as display source:
-    from xpra.x11.gtk_x11.gdk_display_source import init_display_source
-    init_display_source()
+    from xpra.os_util import POSIX
+    if POSIX:
+        from xpra.x11.gtk_x11.gdk_display_source import init_display_source
+        init_display_source()
     from xpra.util import print_nested_dict
     from xpra.platform import program_context
     with program_context("GTK-Keyboard", "GTK Keyboard"):
