@@ -1,11 +1,12 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # Platform-specific code for Win32 -- the parts that may import gtk.
 
+import os
 import sys
 import types
 import ctypes
@@ -17,6 +18,7 @@ screenlog = Logger("win32", "screen")
 keylog = Logger("win32", "keyboard")
 mouselog = Logger("win32", "mouse")
 
+from xpra.client import mixin_features
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.window_hooks import Win32Hooks
 from xpra.platform.win32.win32_events import KNOWN_EVENTS, POWER_EVENTS
@@ -28,32 +30,35 @@ from xpra.platform.win32.common import (GetSystemMetrics, SetWindowLongW, GetWin
                                         GetModuleHandleA,
                                         GetKeyState, GetWindowRect,
                                         GetDoubleClickTime,
-                                        MonitorFromWindow, GetMonitorInfoW, EnumDisplayMonitors,
+                                        MonitorFromWindow, EnumDisplayMonitors,
                                         UnhookWindowsHookEx, CallNextHookEx, SetWindowsHookExA,
                                         SetConsoleCtrlHandler,
                                         GetDeviceCaps,
                                         GetIntSystemParametersInfo,
                                         GetUserObjectInformationA, OpenInputDesktop, CloseDesktop,
+                                        GetMonitorInfo,
                                         user32)
 from xpra.util import AdHocStruct, csv, envint, envbool
-from xpra.os_util import PYTHON2, PYTHON3
+from xpra.os_util import PYTHON2, PYTHON3, get_util_logger
 
 CONSOLE_EVENT_LISTENER = envbool("XPRA_CONSOLE_EVENT_LISTENER", True)
 USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", True)
 REINIT_VISIBLE_WINDOWS = envbool("XPRA_WIN32_REINIT_VISIBLE_WINDOWS", True)
 SCREENSAVER_LISTENER_POLL_DELAY = envint("XPRA_SCREENSAVER_LISTENER_POLL_DELAY", 10)
+APP_ID = os.environ.get("XPRA_WIN32_APP_ID", "Xpra")
 
 
 from ctypes import WinDLL, CFUNCTYPE, c_int, POINTER, Structure, byref, sizeof
-from ctypes.wintypes import HWND, DWORD, WPARAM, LPARAM, MSG, WCHAR, POINT, RECT
+from ctypes.wintypes import HWND, DWORD, WPARAM, LPARAM, MSG, POINT, RECT
 
 if PYTHON3:
     from ctypes import CDLL, pythonapi, c_void_p, py_object
+    from ctypes.util import find_library
     PyCapsule_GetPointer = pythonapi.PyCapsule_GetPointer
     PyCapsule_GetPointer.restype = c_void_p
     PyCapsule_GetPointer.argtypes = [py_object]
     log("PyCapsute_GetPointer=%s", PyCapsule_GetPointer)
-    gdkdll = CDLL("libgdk-3-0.dll")
+    gdkdll = CDLL(find_library("libgdk-3-0.dll"))
     log("gdkdll=%s", gdkdll)
 
 
@@ -75,28 +80,6 @@ except:
     #win XP:
     DwmGetWindowAttribute = None
 
-CCHDEVICENAME = 32
-class MONITORINFOEX(ctypes.Structure):
-    _fields_ = [('cbSize', DWORD),
-                ('rcMonitor', RECT),
-                ('rcWork', RECT),
-                ('dwFlags', DWORD),
-                ('szDevice', WCHAR * CCHDEVICENAME)]
-MONITORINFOEX_size = ctypes.sizeof(MONITORINFOEX)
-def GetMonitorInfo(hmonitor):
-    info = MONITORINFOEX()
-    info.szDevice = ""
-    info.cbSize = MONITORINFOEX_size
-    if not GetMonitorInfoW(hmonitor, byref(info)):
-        raise ctypes.WinError(ctypes.get_last_error())
-    monitor = info.rcMonitor.left, info.rcMonitor.top, info.rcMonitor.right, info.rcMonitor.bottom
-    work = info.rcWork.left, info.rcWork.top, info.rcWork.right, info.rcWork.bottom
-    return  {
-        "Work"      : work,
-        "Monitor"   : monitor,
-        "Flags"     : info.dwFlags,
-        "Device"    : info.szDevice or "",
-        }
 
 WINDOW_HOOKS = envbool("XPRA_WIN32_WINDOW_HOOKS", True)
 GROUP_LEADER = WINDOW_HOOKS and envbool("XPRA_WIN32_GROUP_LEADER", True)
@@ -121,6 +104,8 @@ log("win32 gui settings: CONSOLE_EVENT_LISTENER=%s, USE_NATIVE_TRAY=%s, WINDOW_H
 
 def do_init():
     init_dpi()
+    if APP_ID:
+        init_appid()
 
 def init_dpi():
     #tell win32 we handle dpi
@@ -150,6 +135,15 @@ def init_dpi():
         screenlog("SetProcessDpiAwarenessInternal(%s) failed: %s", DPI_AWARENESS, e)
         screenlog(" (not available on MS Windows before version 8.1)")
 
+def init_appid():
+    from ctypes import HRESULT
+    from ctypes.wintypes import LPCWSTR
+    SetCurrentProcessExplicitAppUserModelID = shell32.SetCurrentProcessExplicitAppUserModelID
+    SetCurrentProcessExplicitAppUserModelID.restype = HRESULT
+    SetCurrentProcessExplicitAppUserModelID.argtypes = [LPCWSTR]
+    if shell32.SetCurrentProcessExplicitAppUserModelID(APP_ID):
+        log.warn("Warning: failed to set process app ID")
+
 
 def get_native_notifier_classes():
     try:
@@ -181,7 +175,13 @@ def get_native_system_tray_classes(*_args):
 def gl_check():
     #This is supposed to help py2exe
     #(must be done after we setup the sys.path in platform.win32.paths):
-    from OpenGL.platform import win32   #@UnresolvedImport @UnusedImport
+    try:
+        from OpenGL.platform import win32   #@UnresolvedImport @UnusedImport
+    except ImportError as e:
+        get_util_logger().warn("gl_check()", exc_info=True)
+        get_util_logger().warn("Warning: OpenGL bindings are missing")
+        get_util_logger().warn(" %s", e)
+        return False
     from xpra.platform.win32 import is_wine
     if is_wine():
         return "disabled when running under wine"
@@ -944,9 +944,9 @@ class ClientExtras(object):
             log.error("Error: cannot register focus and power callbacks:")
             log.error(" %s", e)
         self.keyboard_hook_id = None
-        if FORWARD_WINDOWS_KEY:
-            from xpra.make_thread import make_thread
-            make_thread(self.init_keyboard_listener, "keyboard-listener", daemon=True).start()
+        if FORWARD_WINDOWS_KEY and mixin_features.windows:
+            from xpra.make_thread import start_thread
+            start_thread(self.init_keyboard_listener, "keyboard-listener", daemon=True)
 
     def ready(self):
         pass
@@ -1118,6 +1118,8 @@ class ClientExtras(object):
         log("WM_ACTIVATEAPP: %s/%s client=%s", wParam, lParam, c)
         if not c:
             return
+        if not mixin_features.windows:
+            return
         if wParam==0:
             #our app has lost focus
             c.update_focus(0, False)
