@@ -7,6 +7,7 @@
 # Low level support for the "system tray" on MS Windows
 # Based on code from winswitch, itself based on "win32gui_taskbar demo"
 
+import os
 import ctypes
 from ctypes import POINTER, Structure, byref, WinDLL, c_void_p, sizeof, create_string_buffer
 from ctypes.wintypes import HWND, UINT, POINT, HICON, BOOL, WCHAR, DWORD
@@ -42,7 +43,7 @@ def GetProductInfo(dwOSMajorVersion=5, dwOSMinorVersion=0, dwSpMajorVersion=0, d
     product_type = DWORD(0)
     from xpra.platform.win32.common import GetProductInfo as k32GetProductInfo
     v = k32GetProductInfo(dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, byref(product_type))
-    log("GetProductInfo(%i, %i, %i, %i)=%i product_type=%s", dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, v, product_type)
+    log("GetProductInfo(%i, %i, %i, %i)=%i product_type=%s", dwOSMajorVersion, dwOSMinorVersion, dwSpMajorVersion, dwSpMinorVersion, v, hex(product_type.value))
     return bool(v)
 #win7 is actually 6.1:
 try:
@@ -207,7 +208,7 @@ class win32NotifyIcon(object):
     instances = {}
 
     def __init__(self, app_id=0, title="", move_callbacks=None, click_callback=None, exit_callback=None, command_callback=None, iconPathName=None):
-        log("win32NotifyIcon: app_id=%i, title='%s'", app_id, title)
+        log("win32NotifyIcon: app_id=%i, title='%s'", app_id, nonl(title))
         self.app_id = app_id
         self.title = title
         self.current_icon = None
@@ -217,7 +218,7 @@ class win32NotifyIcon(object):
                 iconPathName = iconPathName.decode()
             except:
                 pass
-            self.current_icon = self.LoadImage(iconPathName)
+            self.current_icon = self.LoadImage(iconPathName) or FALLBACK_ICON
         self.create_tray_window()
         #register callbacks:
         win32NotifyIcon.instances[self.hwnd] = self
@@ -226,8 +227,13 @@ class win32NotifyIcon(object):
         self.exit_callback = exit_callback
         self.command_callback = command_callback
         self.reset_function = None
+        self.image_cache = {}
+
+    def __repr__(self):
+        return "win32NotifyIcon(%#x)" % self.app_id
 
     def create_tray_window(self):
+        log("create_tray_window()")
         self.create_window()
         self.register_tray()
 
@@ -239,7 +245,7 @@ class win32NotifyIcon(object):
             0, 0, NIwc.hInstance, None)
         if self.hwnd==0:
             raise ctypes.WinError(ctypes.get_last_error())
-        log("hwnd=%#x", self.hwnd)
+        log("create_window() hwnd=%#x", self.hwnd)
         UpdateWindow(self.hwnd)
 
     def register_tray(self):
@@ -249,6 +255,7 @@ class win32NotifyIcon(object):
             raise Exception("Shell_NotifyIconW failed to ADD")
 
     def make_nid(self, flags):
+        assert self.hwnd
         nid = NOTIFYICONDATA()
         nid.cbSize = sizeof(NOTIFYICONDATA)
         nid.hWnd = self.hwnd
@@ -280,11 +287,14 @@ class win32NotifyIcon(object):
         try:
             nid = self.make_nid(0)
             log("delete_tray_window(..) calling Shell_NotifyIconW(NIM_DELETE, %s)", nid)
-            Shell_NotifyIconW(NIM_DELETE, nid)
+            if not Shell_NotifyIconW(NIM_DELETE, nid):
+                log.warn("Warning: failed to remove system tray")
+            else:
+                self.hwnd = 0
         except Exception as e:
+            log("delete_tray_window()", exc_info=True)
             log.error("Error: failed to delete tray window")
             log.error(" %s", e)
-        self.hwnd = 0
 
 
     def set_blinking(self, on):
@@ -292,12 +302,14 @@ class win32NotifyIcon(object):
         pass
 
     def set_tooltip(self, tooltip):
+        log("set_tooltip(%s)", tooltip)
         self.title = tooltip
         Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON | NIF_MESSAGE | NIF_TIP))
 
 
     def set_icon(self, iconPathName):
-        hicon = self.LoadImage(iconPathName)
+        log("set_icon(%s)", iconPathName)
+        hicon = self.LoadImage(iconPathName) or FALLBACK_ICON
         self.do_set_icon(hicon)
         Shell_NotifyIconW(NIM_MODIFY, self.make_nid(NIF_ICON))
         self.reset_function = (self.set_icon, iconPathName)
@@ -341,24 +353,30 @@ class win32NotifyIcon(object):
         return image
 
     def doLoadImage(self, iconPathName):
-        v = fallback
-        if iconPathName:
-            icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
-            try:
-                img_type = win32con.IMAGE_ICON
-                if iconPathName.lower().split(".")[-1] in ("png", "bmp"):
-                    img_type = win32con.IMAGE_BITMAP
-                    icon_flags |= win32con.LR_CREATEDIBSECTION | win32con.LR_LOADTRANSPARENT
-                log("LoadImage(%s) using image type=%s", iconPathName,
-                                            {
-                                                win32con.IMAGE_ICON    : "ICON",
-                                                win32con.IMAGE_BITMAP  : "BITMAP",
-                                             }.get(img_type))
-                v = LoadImageW(NIwc.hInstance, iconPathName, img_type, 0, 0, icon_flags)
-            except:
-                log.error("Failed to load icon at %s", iconPathName, exc_info=True)
-        log("LoadImage(%s)=%s", iconPathName, v)
-        return v
+        mingw_prefix = os.environ.get("MINGW_PREFIX")
+        if mingw_prefix and iconPathName.find(mingw_prefix)>=0:
+            #python can deal with mixed win32 and unix paths,
+            #but the native win32 LoadImage function cannot,
+            #this happens when running from a mingw environment
+            iconPathName = iconPathName.replace("/", "\\")
+        icon_flags = win32con.LR_LOADFROMFILE | win32con.LR_DEFAULTSIZE
+        try:
+            img_type = win32con.IMAGE_ICON
+            if iconPathName.lower().split(".")[-1] in ("png", "bmp"):
+                img_type = win32con.IMAGE_BITMAP
+                icon_flags |= win32con.LR_CREATEDIBSECTION | win32con.LR_LOADTRANSPARENT
+            log("LoadImage(%s) using image type=%s", iconPathName,
+                                        {
+                                            win32con.IMAGE_ICON    : "ICON",
+                                            win32con.IMAGE_BITMAP  : "BITMAP",
+                                         }.get(img_type))
+            v = LoadImageW(NIwc.hInstance, iconPathName, img_type, 0, 0, icon_flags)
+        except:
+            log.error("Error: failed to load icon '%s'", iconPathName, exc_info=True)
+            return None
+        else:
+            log("LoadImage(%s)=%#x", iconPathName, v)
+            return v
 
 
     def OnTrayRestart(self, hwnd=0, msg=0, wparam=0, lparam=0):
@@ -389,7 +407,7 @@ class win32NotifyIcon(object):
     def OnTaskbarNotify(self, hwnd, msg, wparam, lparam):
         if lparam==win32con.WM_MOUSEMOVE:
             cb = self.move_callback
-            bm = [(hwnd, msg, wparam, lparam)]
+            bm = [(hwnd, int(msg), int(wparam), int(lparam))]
         else:
             cb = self.click_callback
             bm = BUTTON_MAP.get(lparam)
@@ -432,7 +450,7 @@ message_map = {
 def NotifyIconWndProc(hwnd, msg, wParam, lParam):
     instance = win32NotifyIcon.instances.get(hwnd)
     fn = message_map.get(msg)
-    log("NotifyIconWndProc%s instance=%s, message(%i)=%s", (hwnd, msg, wParam, lParam), instance, msg, fn)
+    log("NotifyIconWndProc%s instance=%s, message(%i)=%s", (int(hwnd), int(msg), int(wParam), int(lParam)), instance, msg, fn)
     #log("potential matching win32 constants for message: %s", [x for x in dir(win32con) if getattr(win32con, x)==msg])
     if instance and fn:
         return fn(instance, hwnd, msg, wParam, lParam) or 0
@@ -453,7 +471,6 @@ log("RegisterClassExA(%s)=%i", NIwc.lpszClassName, NIclassAtom)
 
 
 def main():
-    import os
     from xpra.platform.win32.common import user32
 
     def click_callback(button, pressed):
