@@ -10,14 +10,17 @@ import os
 import sys
 import socket
 import signal
+import binascii
 import threading
 import traceback
 from weakref import WeakKeyDictionary
 from time import sleep
 
 from xpra.log import Logger
+from xpra.platform.paths import get_app_dir
 log = Logger("server")
 netlog = Logger("network")
+ssllog = Logger("ssl")
 httplog = Logger("http")
 wslog = Logger("websocket")
 proxylog = Logger("proxy")
@@ -26,23 +29,18 @@ authlog = Logger("auth")
 timeoutlog = Logger("timeout")
 dbuslog = Logger("dbus")
 
-from xpra.version_util import XPRA_VERSION
-from xpra.version_util import full_version_str
-from xpra.scripts.main import _socket_connect
+from xpra.version_util import XPRA_VERSION, full_version_str, version_compat_check, get_version_info_full, get_platform_info, get_host_info
 from xpra.scripts.server import deadly_signal
-from xpra.scripts.config import InitException, parse_bool, python_platform, parse_with_unit, FALSE_OPTIONS
+from xpra.scripts.config import InitException, parse_bool, python_platform, parse_with_unit, FALSE_OPTIONS, TRUE_OPTIONS
 from xpra.net.bytestreams import SocketConnection, SSLSocketConnection, log_new_connection, pretty_socket, SOCKET_TIMEOUT
 from xpra.net.net_util import get_network_caps, get_info as get_net_info
-from xpra.platform import set_name
-from xpra.os_util import load_binary_file, get_machine_id, get_user_uuid, platform_name, strtobytes, bytestostr, get_hex_uuid, monotonic_time, get_peercred, hexstr, SIGNAMES, WIN32, OSX, POSIX, PYTHON3, BITS
-from xpra.version_util import version_compat_check, get_version_info_full, get_platform_info, get_host_info
 from xpra.net.protocol import Protocol, sanity_checks
-from xpra.net.crypto import crypto_backend_init, new_cipher_caps, get_salt, choose_digest, \
-        ENCRYPTION_CIPHERS, ENCRYPT_FIRST_PACKET, DEFAULT_IV, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING, DEFAULT_PADDING, ALL_PADDING_OPTIONS
+from xpra.net.digest import get_salt, gendigest, choose_digest
+from xpra.platform import set_name
+from xpra.os_util import load_binary_file, get_machine_id, get_user_uuid, platform_name, strtobytes, bytestostr, get_hex_uuid, \
+    getuid, monotonic_time, get_peercred, hexstr, SIGNAMES, WIN32, POSIX, PYTHON3, BITS
 from xpra.server.background_worker import stop_worker, get_worker
 from xpra.make_thread import start_thread
-from xpra.scripts.fdproxy import XpraProxy
-from xpra.server.control_command import ControlError, HelloCommand, HelpCommand, DebugControl
 from xpra.util import csv, merge_dicts, typedict, notypedict, flatten_dict, parse_simple_dict, repr_ellipsized, dump_all_frames, nonl, envint, envbool, envfloat, \
         SERVER_SHUTDOWN, SERVER_UPGRADE, LOGIN_TIMEOUT, DONE, PROTOCOL_ERROR, SERVER_ERROR, VERSION_ERROR, CLIENT_REQUEST, SERVER_EXIT
 
@@ -207,6 +205,7 @@ class ServerCore(object):
         self.dbus_control = False
         self.dbus_server = None
         self.unix_socket_paths = []
+        self.touch_timer = None
 
         self.session_name = u""
 
@@ -257,6 +256,7 @@ class ServerCore(object):
         self.tcp_encryption = opts.tcp_encryption
         self.tcp_encryption_keyfile = opts.tcp_encryption_keyfile
         if self.encryption or self.tcp_encryption:
+            from xpra.net.crypto import crypto_backend_init
             crypto_backend_init()
         self.password_file = opts.password_file
         self.compression_level = opts.compression_level
@@ -264,6 +264,7 @@ class ServerCore(object):
         self.server_idle_timeout = opts.server_idle_timeout
         self.readonly = opts.readonly
         self.ssl_mode = opts.ssl
+        self.ssh_upgrade = opts.ssh_upgrade
         self.dbus_control = opts.dbus_control
         self.init_html_proxy(opts)
         self.init_auth(opts)
@@ -351,6 +352,7 @@ class ServerCore(object):
         self.idle_add(self.reset_server_timeout)
         self.idle_add(self.server_is_ready)
         self.do_run()
+        log("run() returning %s", self._upgrading)
         return self._upgrading
 
     def server_is_ready(self):
@@ -365,6 +367,7 @@ class ServerCore(object):
         for p in tuple(self._tcp_proxy_clients):
             p.quit()
         netlog("cleanup will disconnect: %s", self._potential_protocols)
+        self.cancel_touch_timer()
         if self._upgrading:
             reason = SERVER_UPGRADE
         else:
@@ -443,7 +446,7 @@ class ServerCore(object):
         httplog("init_html_proxy(..) html=%s", self._html)
         if self._html is not False:
             try:
-                from xpra.net.websocket import WebSocketConnection
+                from xpra.server.websocket import WebSocketConnection
                 assert WebSocketConnection
                 self._html = True
             except ImportError as e:
@@ -457,7 +460,18 @@ class ServerCore(object):
                 self._html = False
         #make sure we have the web root:
         from xpra.platform.paths import get_resources_dir
-        self._www_dir = www_dir or os.path.abspath(os.path.join(get_resources_dir(), "www"))
+        if www_dir:
+            self._www_dir = www_dir
+        else:
+            for ad,d in (
+                (get_resources_dir(), "html5"),
+                (get_resources_dir(), "www"),
+                (get_app_dir(), "www"),
+                ):
+                self._www_dir = os.path.abspath(os.path.join(ad, d))
+                if os.path.exists(self._www_dir):
+                    httplog("found html5 client in '%s'", self._www_dir)
+                    break
         self._http_headers_dir = os.path.abspath(os.path.join(self._www_dir, "../http-headers"))
         if not os.path.exists(self._www_dir) and self._html:
             httplog.error("Error: cannot find the html web root")
@@ -479,7 +493,7 @@ class ServerCore(object):
             self.auth_classes["named-pipe"] = auth
         else:
             self.auth_classes["unix-domain"] = auth
-        for x in ("tcp", "ws", "wss", "ssl", "rfb", "vsock", "udp"):
+        for x in ("tcp", "ws", "wss", "ssl", "ssh", "rfb", "vsock", "udp"):
             opts_value = getattr(opts, "%s_auth" % x)
             self.auth_classes[x] = self.get_auth_modules(x, opts_value, opts)
         authlog("init_auth(..) auth=%s", self.auth_classes)
@@ -501,93 +515,37 @@ class ServerCore(object):
         if len(parts)>1:
             auth_options = parse_simple_dict(parts[1])
         auth_options["exec_cwd"] = self.exec_cwd
-        from xpra.server.auth import fail_auth, reject_auth, allow_auth, none_auth, file_auth, multifile_auth, password_auth, env_auth, exec_auth
-        AUTH_MODULES = {
-                        "fail"      : fail_auth,
-                        "reject"    : reject_auth,
-                        "allow"     : allow_auth,
-                        "none"      : none_auth,
-                        "env"       : env_auth,
-                        "password"  : password_auth,
-                        "multifile" : multifile_auth,
-                        "file"      : file_auth,
-                        "exec"      : exec_auth,
-                        }
         try:
-            from xpra.server.auth import ldap_auth
-            AUTH_MODULES["ldap"] = ldap_auth
-        except ImportError:
-            authlog("cannot load ldap auth: %s", exc_info=True)
-        try:
-            from xpra.server.auth import ldap3_auth
-            AUTH_MODULES["ldap3"] = ldap3_auth
-        except ImportError:
-            authlog("cannot load ldap3 auth: %s", exc_info=True)
-        try:
-            from xpra.server.auth import u2f_auth
-            AUTH_MODULES["u2f"] = u2f_auth
-        except ImportError:
-            authlog("cannot load u2f auth: %s", exc_info=True)
-        if POSIX and not OSX:
-            from xpra.server.auth import peercred_auth, hosts_auth
-            AUTH_MODULES["peercred"] = peercred_auth
-            AUTH_MODULES["hosts"] = hosts_auth
-        try:
-            from xpra.server.auth import sqlite_auth
-            AUTH_MODULES["sqlite"] = sqlite_auth
-        except Exception:
-            authlog("cannot load sql auth: %s", exc_info=True)
-        try:
-            from xpra.server.auth import kerberos_password_auth
-            AUTH_MODULES["kerberos-password"] = kerberos_password_auth
-        except Exception as e:
-            authlog("cannot load kerberos-password auth", exc_info=True)
-        try:
-            from xpra.server.auth import kerberos_token_auth
-            AUTH_MODULES["kerberos-token"] = kerberos_token_auth
-        except Exception:
-            authlog("cannot load kerberos-token auth", exc_info=True)
-        try:
-            from xpra.server.auth import gss_auth
-            AUTH_MODULES["gss"] = gss_auth
-        except Exception as e:
-            authlog("cannot load kerberos-token auth", exc_info=True)
-        if WIN32:
-            try:
-                from xpra.server.auth import win32_auth
-                AUTH_MODULES["win32"] = win32_auth
-            except Exception as e:
-                authlog("cannot load win32 auth", exc_info=True)
-                authlog.error("Error: cannot load the MS Windows authentication module:")
-                authlog.error(" %s", e)
-        else:
-            try:
-                from xpra.server.auth import pam_auth
-                AUTH_MODULES["pam"] = pam_auth
-            except Exception:
-                authlog("cannot load pam auth", exc_info=True)
-        if auth=="sys":
-            #resolve virtual "sys" auth:
-            if WIN32:
-                auth = "win32"
+            if auth=="sys":
+                #resolve virtual "sys" auth:
+                if WIN32:
+                    auth_modname = "win32_auth"
+                else:
+                    auth_modname = "pam_auth"
+                authlog("will try to use sys auth module '%s' for %s", auth, sys.platform)
             else:
-                auth = "pam"
-            authlog("will try to use sys auth module '%s' for %s", auth, sys.platform)
-        auth_module = AUTH_MODULES.get(auth)
-        if not auth_module:
-            raise InitException("cannot find authentication module '%s' (supported: %s)" % (auth, csv(AUTH_MODULES.keys())))
+                auth_modname = auth.replace("-", "_")+"_auth"
+            auth_mod_name = "xpra.server.auth."+auth_modname
+            authlog("auth module name for '%s': '%s'", auth, auth_mod_name)
+            auth_module = __import__(auth_mod_name, {}, {}, ["Authenticator"])
+        except ImportError as e:
+            authlog("cannot load %s auth for socket %s", auth, socket_type, exc_info=True)
+            raise InitException("cannot load authentication module '%s' for %s socket: %s" % (auth, socket_type, e))
+        authlog("auth module for '%s': %s", auth, auth_module)
         try:
             auth_module.init(opts)
-            auth_class = getattr(auth_module, "Authenticator")
+            auth_class = auth_module.Authenticator
             auth_class.auth_name = auth.lower()
             return auth, auth_class, auth_options
         except Exception as e:
-            raise InitException("Authenticator class not found in %s" % auth_module)
+            authlog("cannot access authenticator class", exc_info=True)
+            raise InitException("authentication setup error in %s: %s" % (auth_module, e))
 
 
     ######################################################################
     # control commands:
     def init_control_commands(self):
+        from xpra.server.control_command import HelloCommand, HelpCommand, DebugControl
         self.control_commands = {"hello"    : HelloCommand(),
                                  "debug"    : DebugControl()}
         help_command = HelpCommand(self.control_commands)
@@ -601,6 +559,7 @@ class ServerCore(object):
         proto.send_now(("hello", hello))
 
     def process_control_command(self, *args):
+        from xpra.server.control_command import ControlError
         assert len(args)>0
         name = args[0]
         try:
@@ -645,6 +604,9 @@ class ServerCore(object):
                 log.info(" uid=%i, gid=%i", uid, gid)
         log.info(" running with pid %s%s", os.getpid(), osinfo)
 
+    def notify_new_user(self, ss):
+        pass
+
 
     ######################################################################
     # screen / display:
@@ -678,6 +640,24 @@ class ServerCore(object):
                 except Exception as e:
                     log.error("failed to set socket path to %s: %s", info, e)
                     del e
+        if self.unix_socket_paths:
+            self.touch_timer = self.timeout_add(60*1000, self.touch_sockets)
+
+
+    def cancel_touch_timer(self):
+        tt = self.touch_timer
+        if tt:
+            self.touch_timer = None
+            self.source_remove(tt)
+
+    def touch_sockets(self):
+        netlog("touch_sockets() unix socket paths=%s", self.unix_socket_paths)
+        for sockpath in self.unix_socket_paths:
+            try:
+                os.utime(sockpath, None)
+            except Exception:
+                netlog("touch_sockets() error on %s", sockpath, exc_info=True)
+        return True
 
     def init_packet_handlers(self):
         netlog("initializing packet handlers")
@@ -737,10 +717,13 @@ class ServerCore(object):
                 udpl = UDPListener(sock, self.process_udp_packet)
                 self._udp_listeners.append(udpl)
             else:
-                from xpra.gtk_common.gobject_compat import import_glib
+                from xpra.gtk_common.gobject_compat import import_glib, is_gtk3
                 glib = import_glib()
                 sock.listen(5)
-                glib.io_add_watch(sock, glib.IO_IN, self._new_connection, sock)
+                if is_gtk3():
+                    glib.io_add_watch(sock, glib.PRIORITY_DEFAULT, glib.IO_IN, self._new_connection, sock)
+                else:
+                    glib.io_add_watch(sock, glib.IO_IN, self._new_connection, sock, priority=glib.PRIORITY_DEFAULT)
         except Exception as e:
             netlog("add_listen_socket(%s, %s)", socktype, sock, exc_info=True)
             netlog.error("Error: failed to listen on %s socket %s:", socktype, info or sock)
@@ -808,7 +791,7 @@ class ServerCore(object):
         if peek_data:
             line1 = peek_data.splitlines()[0]
             netlog("socket peek=%s", repr_ellipsized(peek_data, limit=512))
-            netlog("socket peek hex=%s", hexstr(peek_data))
+            netlog("socket peek hex=%s", hexstr(peek_data[:128]))
             netlog("socket peek line1=%s", repr_ellipsized(line1))
         return peek_data, line1
 
@@ -853,22 +836,20 @@ class ServerCore(object):
         netlog("handle_new_connection%s sockname=%s, target=%s", (conn, sock, address, socktype, peername, socket_info), sockname, target)
         #peek so we can detect invalid clients early,
         #or handle non-xpra traffic:
-        if socktype=="rfb":
-            #rfb does not send any data, waits for a server packet
-            poll_timeout = 0
-        else:
-            poll_timeout = PEEK_TIMEOUT
-        peek_data, line1 = self.peek_connection(conn, poll_timeout)
+        peek_data, line1 = None, None
+        #rfb does not send any data, waits for a server packet
+        if socktype!="rfb":
+            peek_data, line1 = self.peek_connection(conn, PEEK_TIMEOUT)
 
         def ssl_wrap():
             ssl_sock = self._ssl_wrap_socket(sock)
-            netlog("ssl wrapped socket(%s)=%s", sock, ssl_sock)
+            ssllog("ssl wrapped socket(%s)=%s", sock, ssl_sock)
             if ssl_sock is None:
                 #None means EOF! (we don't want to import ssl bits here)
-                netlog("ignoring SSL EOF error")
+                ssllog("ignoring SSL EOF error")
                 return None
             ssl_conn = SSLSocketConnection(ssl_sock, sockname, address, target, socktype)
-            netlog("ssl_wrap()=%s", ssl_conn)
+            ssllog("ssl_wrap()=%s", ssl_conn)
             return ssl_conn
 
         if socktype=="ssl" or socktype=="wss":
@@ -890,7 +871,7 @@ class ServerCore(object):
             http = False
             if socktype=="wss" or self.ssl_mode=="www":
                 http = True
-            elif self.ssl_mode=="auto":
+            elif self.ssl_mode=="auto" or self.ssl_mode in TRUE_OPTIONS:
                 #look for HTTPS request to handle:
                 if line1.find("HTTP/")>0 or peek_data.find("\x08http/1.1")>0:
                     http = True
@@ -898,7 +879,7 @@ class ServerCore(object):
                     ssl_conn.enable_peek()
                     peek_data, line1 = self.peek_connection(ssl_conn, PEEK_TIMEOUT)
                     http = line1.find("HTTP/")>0
-            if http:
+            if http and self._html:
                 self.start_http_socket(socktype, ssl_conn, True, peek_data)
             else:
                 log_new_connection(conn, socket_info)
@@ -911,7 +892,7 @@ class ServerCore(object):
                     if not self._ssl_wrap_socket:
                         netlog.warn("Warning: cannot upgrade to SSL socket")
                         return None
-                    netlog("ws socket receiving ssl, upgrading")
+                    ssllog("ws socket receiving ssl, upgrading")
                     conn = ssl_wrap()
                 elif len(peek_data)>=2 and peek_data[0] in ("P", ord("P") and peek_data[1] in ("\x00", 0)):
                     self.new_conn_err(conn, sock, socktype, socket_info, "xpra", "packet looks like a plain xpra packet")
@@ -923,12 +904,19 @@ class ServerCore(object):
             self.handle_rfb_connection(conn)
             return
 
-        elif (socktype=="tcp" and (self._tcp_proxy or self._ssl_wrap_socket)) or \
+        elif socktype=="ssh":
+            conn = self.handle_ssh_connection(conn)
+            if not conn:
+                return
+            peek_data = None
+
+        elif (socktype=="tcp" and (self._tcp_proxy or self._ssl_wrap_socket or self.ssh_upgrade)) or \
             (socktype in ("tcp", "unix-domain", "named-pipe") and self._html):
             #see if the packet data is actually xpra or something else
             #that we need to handle via a tcp proxy, ssl wrapper or the websockify adapter:
             try:
                 cont, conn, peek_data = self.may_wrap_socket(conn, socktype, peek_data, line1)
+                netlog("may_wrap_socket(..)=(%s, %s, %r)", cont, conn, peek_data)
                 if not cont:
                     return
             except IOError as e:
@@ -950,6 +938,44 @@ class ServerCore(object):
             t = self.timeout_add(self._rfb_upgrade*1000, self.try_upgrade_to_rfb, proto)
             self.socket_rfb_upgrade_timer[proto] = t
 
+    def handle_ssh_connection(self, conn):
+        from xpra.server.ssh import make_ssh_server_connection, log as sshlog
+        socktype = conn.socktype_wrapped
+        sshlog("handle_ssh_connection(%s) socktype wrapped=%s", conn, socktype)
+        def ssh_password_authenticate(username, password):
+            if not POSIX or getuid()!=0:
+                import getpass
+                sysusername = getpass.getuser()
+                if sysusername!=username:
+                    sshlog.warn("Warning: ssh password authentication failed,")
+                    sshlog.warn(" username does not match:")
+                    sshlog.warn(" expected '%s', got '%s'", sysusername, username)
+                    return False
+            auth_modules = self.make_authenticators(socktype, username, conn)
+            sshlog("ssh_password_authenticate auth_modules(%s, %s)=%s", username, "*"*len(password), auth_modules)
+            for auth in auth_modules:
+                #mimic a client challenge:
+                digests = ["xor"]
+                try:
+                    salt, digest = auth.get_challenge(digests)
+                    salt_digest = auth.choose_salt_digest(digests)
+                    assert digest=="xor" and salt_digest=="xor"
+                except ValueError as e:
+                    sshlog("authentication with %s", auth, exc_info=True)
+                    sshlog.warn("Warning: ssh transport cannot use %r authentication:", auth)
+                    sshlog.warn(" %s", e)
+                    return False
+                else:
+                    client_salt = get_salt(len(salt))
+                    combined_salt = gendigest("xor", client_salt, salt)
+                    xored_password = gendigest("xor", password, combined_salt)
+                    r = auth.authenticate(xored_password, client_salt)
+                    sshlog("%s.authenticate(..)=%s", auth, r)
+                    if not r:
+                        return False
+            return True
+        return make_ssh_server_connection(conn, ssh_password_authenticate)
+
     def try_upgrade_to_rfb(self, proto):
         if proto._closed:
             return
@@ -977,7 +1003,7 @@ class ServerCore(object):
     def make_protocol(self, socktype, conn):
         def xpra_protocol_class(conn):
             protocol = Protocol(self, conn, self.process_packet)
-            protocol.large_packets.append("info-response")
+            protocol.large_packets.append(b"info-response")
             protocol.receive_aliases.update(self._aliases)
             return protocol
         return self.do_make_protocol(socktype, conn, xpra_protocol_class)
@@ -995,10 +1021,12 @@ class ServerCore(object):
             #special case for legacy encryption code:
             protocol.encryption = self.tcp_encryption
             protocol.keyfile = self.tcp_encryption_keyfile
-            if protocol.encryption and ENCRYPT_FIRST_PACKET:
-                authlog("encryption=%s, keyfile=%s", protocol.encryption, protocol.keyfile)
-                password = self.get_encryption_key(None, protocol.keyfile)
-                protocol.set_cipher_in(protocol.encryption, DEFAULT_IV, password, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING)
+            if protocol.encryption:
+                from xpra.net.crypto import ENCRYPT_FIRST_PACKET, DEFAULT_IV, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING
+                if ENCRYPT_FIRST_PACKET:
+                    authlog("encryption=%s, keyfile=%s", protocol.encryption, protocol.keyfile)
+                    password = self.get_encryption_key(None, protocol.keyfile)
+                    protocol.set_cipher_in(protocol.encryption, DEFAULT_IV, password, DEFAULT_SALT, DEFAULT_ITERATIONS, INITIAL_PADDING)
         protocol.invalid_header = self.invalid_header
         authlog("socktype=%s, encryption=%s, keyfile=%s", socktype, protocol.encryption, protocol.keyfile)
         protocol.start()
@@ -1021,22 +1049,27 @@ class ServerCore(object):
             #xpra packet header, no need to wrap this connection
             return True, conn, peek_data
         frominfo = pretty_socket(conn.remote)
-        if self._ssl_wrap_socket and peek_data[0] in (chr(0x16), 0x16):
+        netlog("may_wrap_socket(..) peek_data=%s from %s", binascii.hexlify(peek_data), frominfo)
+        if self.ssh_upgrade and peek_data[:4]==b"SSH-":
+            conn = self.handle_ssh_connection(conn)
+            return conn!=None, conn, None
+        elif self._ssl_wrap_socket and peek_data[0] in (chr(0x16), 0x16):
             sock, sockname, address, endpoint = conn._socket, conn.local, conn.remote, conn.endpoint
             sock = self._ssl_wrap_socket(sock)
             if sock is None:
                 #None means EOF! (we don't want to import ssl bits here)
-                netlog("ignoring SSL EOF error")
+                ssllog("ignoring SSL EOF error")
                 return False, None, None
             conn = SSLSocketConnection(sock, sockname, address, endpoint, "ssl")
             conn.socktype_wrapped = socktype
             #we cannot peek on SSL sockets, just clear the unencrypted data:
-            netlog("may_wrap_socket SSL: %s, ssl mode=%s", conn, self.ssl_mode)
+            ssllog("may_wrap_socket SSL: %s, ssl mode=%s", conn, self.ssl_mode)
+            http = False
             if self.ssl_mode=="tcp":
                 http = False
             elif self.ssl_mode=="www":
                 http = True
-            else:   #ie: auto
+            elif self.ssl_mode=="auto" or self.ssl_mode in TRUE_OPTIONS:
                 http = False
                 #use the header to guess:
                 if line1.find(b"HTTP/")>0 or peek_data.find(b"\x08http/1.1")>0:
@@ -1071,7 +1104,6 @@ class ServerCore(object):
             c = ord(v[0])
         except:
             c = int(v[0])
-        import binascii
         netlog("guess_header_protocol(%s)", binascii.hexlify(strtobytes(v)))
         if c==0x16:
             return "ssl", "SSL packet?"
@@ -1097,6 +1129,7 @@ class ServerCore(object):
         if peek_data:
             line1 = peek_data.splitlines()[0]
         http_proto = "http"+["","s"][int(is_ssl)]
+        netlog("start_http_socket(%s, %s, %s, ..) http proto=%s, line1=%r", socktype, conn, is_ssl, http_proto, line1)
         if line1.startswith(b"GET ") or line1.startswith(b"POST "):
             parts = bytestostr(line1).split(" ")
             httplog("New %s %s request received from %s for '%s'", http_proto, parts[0], frominfo, parts[1])
@@ -1112,7 +1145,7 @@ class ServerCore(object):
 
     def start_websockify(self, socktype, conn, is_ssl, req_info, frominfo):
         wslog("start_websockify(%s, %s, %s, %s, %s) www dir=%s, headers dir=%s", socktype, conn, is_ssl, req_info, frominfo, self._www_dir, self._http_headers_dir)
-        from xpra.net.websocket import WebSocketConnection, WSRequestHandler
+        from xpra.server.websocket import WebSocketConnection, WSRequestHandler
         try:
             sock = conn._socket
             sock.settimeout(self._ws_timeout)
@@ -1181,18 +1214,27 @@ class ServerCore(object):
     def start_tcp_proxy(self, conn, frominfo):
         proxylog("start_tcp_proxy(%s, %s)", conn, frominfo)
         #connect to web server:
-        host, port = self._tcp_proxy.split(":", 1)
+        try:
+            host, port = self._tcp_proxy.split(":", 1)
+            port = int(port)
+        except ValueError as e:
+            proxylog.error("Error: invalid tcp proxy value '%s'", self._tcp_proxy)
+            proxylog.error(" %s", e)
+            conn.close()
+            return
         try:
             sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             sock.settimeout(10)
-            host, port = self._tcp_proxy.split(":", 1)
-            tcp_server_connection = _socket_connect(sock, (host, int(port)), "web-proxy-for-%s" % frominfo, "tcp")
-        except:
-            proxylog.warn("failed to connect to proxy: %s:%s", host, port)
+            sock.connect((host, int(port)))
+            sock.settimeout(None)
+            tcp_server_connection = SocketConnection(sock, sock.getsockname(), sock.getpeername(), "tcp-proxy-for-%s" % frominfo, "tcp")
+        except Exception as e:
+            proxylog("start_tcp_proxy(%s, %s)", conn, frominfo, exc_info=True)
+            proxylog.error("Error: failed to connect to TCP proxy endpoint: %s:%s", host, port)
+            proxylog.error(" %s", e)
             conn.close()
             return
         proxylog("proxy connected to tcp server at %s:%s : %s", host, port, tcp_server_connection)
-        sock = tcp_server_connection._socket
         sock.settimeout(self._socket_timeout)
 
         #we can use blocking sockets for the client:
@@ -1201,6 +1243,7 @@ class ServerCore(object):
         sock.settimeout(1)
 
         #now start forwarding:
+        from xpra.scripts.fdproxy import XpraProxy
         p = XpraProxy(frominfo, conn, tcp_server_connection, self.tcp_proxy_quit)
         self._tcp_proxy_clients.append(p)
         proxylog.info("client connection from %s forwarded to proxy server on %s:%s", frominfo, host, port)
@@ -1227,7 +1270,7 @@ class ServerCore(object):
         self.cancel_verify_connection_accepted(protocol)
         if self.is_timedout(protocol):
             info = getattr(protocol, "_conn", protocol)
-            log.error("connection timedout: %s", info)
+            log.error("Error: connection timed out: %s", info)
             self.send_disconnect(protocol, LOGIN_TIMEOUT)
 
     def cancel_verify_connection_accepted(self, protocol):
@@ -1409,11 +1452,12 @@ class ServerCore(object):
         cipher = c.strget("cipher")
         cipher_iv = c.strget("cipher.iv")
         key_salt = c.strget("cipher.key_salt")
-        iterations = c.intget("cipher.key_stretch_iterations")
-        padding = c.strget("cipher.padding", DEFAULT_PADDING)
-        padding_options = c.strlistget("cipher.padding.options", [DEFAULT_PADDING])
         auth_caps = {}
         if cipher and cipher_iv:
+            from xpra.net.crypto import DEFAULT_PADDING, ALL_PADDING_OPTIONS, ENCRYPTION_CIPHERS, new_cipher_caps
+            iterations = c.intget("cipher.key_stretch_iterations")
+            padding = c.strget("cipher.padding", DEFAULT_PADDING)
+            padding_options = c.strlistget("cipher.padding.options", [DEFAULT_PADDING])
             if cipher not in ENCRYPTION_CIPHERS:
                 authlog.warn("unsupported cipher: %s", cipher)
                 auth_failed("unsupported cipher")
@@ -1583,7 +1627,7 @@ class ServerCore(object):
             ctr = c.strget("connect_test_request")
             response = {"connect_test_response" : ctr}
             proto.send_now(("hello", response))
-            return
+            return True
         if is_req("id"):
             self.send_id_info(proto)
             return True
@@ -1642,10 +1686,11 @@ class ServerCore(object):
                         "server.mode"           : self.get_server_mode(),
                         })
         if source is None or source.wants_features:
-            capabilities["info-request"] = True
-            capabilities["readonly-server"] = True
-            if self.readonly:
-                capabilities["readonly"] = True
+            capabilities.update({
+                "info-request"      : True,
+                "readonly-server"   : True,
+                "readonly"          : self.readonly,
+                })
         if source is None or source.wants_versions:
             capabilities["uuid"] = get_user_uuid()
             mid = get_machine_id()
@@ -1841,7 +1886,7 @@ class ServerCore(object):
             def udp_protocol_class(conn):
                 protocol = UDPServerProtocol(self, conn, self.process_packet)
                 protocol.uuid = uuid
-                protocol.large_packets.append("info-response")
+                protocol.large_packets.append(b"info-response")
                 protocol.receive_aliases.update(self._aliases)
                 return protocol
             socktype = "udp"
